<?php

declare(strict_types=1);

namespace Drupal\search_api_typesense_ai;

use Drupal\Core\Form\FormStateInterface;
use Drupal\search_api_typesense\Entity\TypesenseSchema;

/**
 * Interface for embedding_model plugins.
 */
interface EmbeddingModelInterface {

  /**
   * Returns the translated plugin label.
   *
   * @return string
   *   The translated plugin label.
   */
  public function label(): string;

  /**
   * Returns the model configuration.
   *
   * @param \Drupal\search_api_typesense\Entity\TypesenseSchema $schema
   *   The schema.
   *
   * @return array
   *   The model configuration.
   */
  public function buildConfiguration(TypesenseSchema $schema): array;

  /**
   * Weather the model is remote or not.
   *
   * @return bool
   *   True if the model is remote, false otherwise.
   */
  public function isRemote(): bool;

  /**
   * Build the configuration form.
   *
   * @param array $form
   *   The form array.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The form state.
   * @param \Drupal\search_api_typesense\Entity\TypesenseSchema $schema
   *   The schema.
   *
   * @return array
   *   The form array.
   */
  public function buildConfigurationForm(
    array $form,
    FormStateInterface $form_state,
    TypesenseSchema $schema,
  ): array;

}
