<?php

declare(strict_types=1);

namespace Drupal\search_api_typesense_ai\EventSubscriber;

use Drupal\search_api_typesense\Event\TypesenseSchemaEvent;
use Drupal\search_api_typesense\Event\TypesenseSchemaEvents;
use Drupal\search_api_typesense_ai\EmbeddingModelPluginManager;
use Symfony\Component\EventDispatcher\EventSubscriberInterface;

/**
 * Event subscriber for altering the Typesense schema.
 */
final class TypesenseSchemaAlterSubscriber implements EventSubscriberInterface {

  /**
   * TypesenseSchemaAlterSubscriber constructor.
   *
   * @param \Drupal\search_api_typesense_ai\EmbeddingModelPluginManager $embeddingModelManager
   *   Embedding model plugin manager.
   */
  public function __construct(
    private readonly EmbeddingModelPluginManager $embeddingModelManager,
  ) {}

  /**
   * Kernel request event handler.
   */
  public function onTypesenseSchemaAlter(TypesenseSchemaEvent $event): void {
    $schema = $event->getSchema();
    $generated_schema = $event->getGeneratedSchema();

    $enable_embedding = $schema
      ->getThirdPartySetting(
        module: 'search_api_typesense_ai',
        key   : 'enable_embedding',
      );

    if ($enable_embedding === 1) {
      $embedding_fields = $schema
        ->getThirdPartySetting(
          module: 'search_api_typesense_ai',
          key   : 'embedding_fields',
        );

      $embedding_model = $schema
        ->getThirdPartySetting(
          module: 'search_api_typesense_ai',
          key   : 'embedding_model',
        );

      /** @var \Drupal\search_api_typesense_ai\EmbeddingModelInterface $embedding_model_instance */
      $embedding_model_instance = $this->embeddingModelManager->createInstance($embedding_model);

      $generated_schema['fields'][] = [
        'name' => 'embedding',
        'type' => 'float[]',
        'embed' => [
          'from' => \array_keys(\array_filter($embedding_fields,
            static fn($field) => $field != 0)),
          'model_config' => $embedding_model_instance->buildConfiguration($schema),
        ],
      ];

      $event->setGeneratedSchema($generated_schema);
    }
  }

  /**
   * {@inheritdoc}
   */
  public static function getSubscribedEvents(): array {
    return [
      TypesenseSchemaEvents::ALTER_SCHEMA => ['onTypesenseSchemaAlter'],
    ];
  }

}
