<?php

declare(strict_types=1);

namespace Drupal\search_api_typesense_ai\Plugin\search_api_typesense_ai\EmbeddingModel;

use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use Drupal\search_api_typesense\Entity\TypesenseSchema;
use Drupal\search_api_typesense_ai\EmbeddingModelPluginBase;

/**
 * Describe OpenAI embedding models.
 */
abstract class OpenAI3Base extends EmbeddingModelPluginBase {

  use StringTranslationTrait;

  /**
   * {@inheritdoc}
   */
  public function buildConfiguration(TypesenseSchema $schema): array {
    $embedding_model_config = $schema
      ->getThirdPartySetting(
        module: 'search_api_typesense_ai',
        key   : 'embedding_model_config',
      );

    return [
      'model_name' => $this->pluginDefinition['name'],
      'api_key' => $embedding_model_config != NULL ? $embedding_model_config['api_key'] : '',
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function buildConfigurationForm(
    array $form,
    FormStateInterface $form_state,
    TypesenseSchema $schema,
  ): array {
    $form = [];

    $embedding_model_config = $schema
      ->getThirdPartySetting(
        module: 'search_api_typesense_ai',
        key   : 'embedding_model_config',
      );

    $form['api_key'] = [
      '#type' => 'textfield',
      '#title' => $this->t('API Key'),
      '#default_value' => $embedding_model_config != NULL ? $embedding_model_config['api_key'] : '',
      '#required' => TRUE,
    ];

    return $form;
  }

}
