<?php

declare(strict_types=1);

namespace Drupal\search_api_typesense\Form;

use Drupal\search_api\IndexInterface;
use Drupal\search_api_typesense\Plugin\search_api\backend\SearchApiTypesenseBackend;

/**
 * Provides a trait for backend related methods.
 */
trait BackendTrait {

  /**
   * Return the Typesense backend.
   *
   * @param \Drupal\search_api\IndexInterface|null $search_api_index
   *   The search API index.
   *
   * @return \Drupal\search_api_typesense\Plugin\search_api\backend\SearchApiTypesenseBackend
   *   The Typesense backend.
   *
   * @throws \Drupal\search_api\SearchApiException
   * @throws \Drupal\search_api_typesense\Api\SearchApiTypesenseException
   */
  private function getBackend(
    ?IndexInterface $search_api_index,
  ): SearchApiTypesenseBackend {
    $search_api_server = $search_api_index->getServerInstance();
    $backend = $search_api_server->getBackend();
    if (!$backend instanceof SearchApiTypesenseBackend) {
      throw new \InvalidArgumentException('The server must use the Typesense backend.');
    }

    if (!$backend->isAvailable()) {
      throw new \InvalidArgumentException('The Typesense server is not available.');
    }

    $collection = $backend->getTypesenseClient()
      ->retrieveCollectionInfo($search_api_index->id());
    if ($collection == NULL) {
      throw new \InvalidArgumentException('The collection does not exist.');
    }

    return $backend;
  }

}
